/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.impl;

import java.util.* ;
import java.util.jar.* ;
import java.io.* ;
import org.exoplatform.commons.utils.IOUtil;
import org.exoplatform.services.backup.ImporterExporter;
import org.exoplatform.services.portal.model.*;

import com.thoughtworks.xstream.XStream;

import net.sf.hibernate.Hibernate;
import net.sf.hibernate.Session;

/**
 * May 27, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $ID$
 **/
public class PortalConfigImporterExporter implements ImporterExporter {
  private XStream xstream_ ; 
  
  public PortalConfigImporterExporter() {
    xstream_ = PortalConfigServiceImpl.getXStreamInstance() ;
  }
  
  public String getName() { return  "portal-config" ;}
  
  public String getDescription() {
    return "Export  portal config and pages" ;
  }
  
  public void exportUserData(String userName, Object transaction,
                             OutputStream out, JarEntry entry) throws Exception {
    Session session = (Session) transaction;
    PortalConfigData data = 
      (PortalConfigData) session.get(PortalConfigData.class, userName) ;
    PortalConfig pconfig = data.getPortalConfig() ;
    List l = session.find(PortalConfigServiceImpl.queryPageDataByOwner, userName, Hibernate.STRING) ;
    List pages = new ArrayList(l.size())  ;
    for(int i = 0 ; i < l.size(); i++) {
      PageData pageData = (PageData) l.get(i) ;
      pages.add(pageData.getPage()) ;
    }
    Backup backup = new Backup(pconfig, pages, null) ;
    String xml = xstream_.toXML(backup) ;
    out.write(xml.getBytes()) ;
  }
  
  public void importUserData(String userName, Object transaction, 
                             InputStream is, JarEntry entry) throws Exception {
    Session session = (Session) transaction;
    String xml = IOUtil.getStreamContentAsString(is) ;
    Backup backup = (Backup)xstream_.fromXML(xml) ;
    PortalConfig pconfig = backup.getPortalConfig() ;
    List pages = backup.getPages() ;
    session.saveOrUpdateCopy(new PortalConfigData(pconfig)) ;
    for(int i = 0; i < pages.size(); i++) {
      Page page = (Page)pages.get(i) ;
      session.saveOrUpdateCopy(new PageData(page)) ;
    }
  }
  
  public void exportServiceData(Object transaction, JarOutputStream out) throws Exception {
  	 JarEntry entry = new JarEntry("data.xml") ;
     out.putNextEntry(entry) ;
     out.write("no data available".getBytes());
     out.closeEntry() ;
  }
  
  public void importServiceData(Object transaction, JarFile jar) throws Exception {
    
  }
}